<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Exception;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Log;

class PNRAddMultiElementsSecondAction extends AmadeusRequestInSeries
{
    use FileStorageTrait;

    protected int $sequenceNumber;
    protected string $amadeusSessionId;
    protected string $amadeusSecurityToken;
    private array $passengerCriteria;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 6;
    }

    public function execute( string $amadeusSessionId, string $amadeusSecurityToken): array
    {
        try {
            $this->amadeusSessionId = $amadeusSessionId;
            $this->amadeusSecurityToken = $amadeusSecurityToken;

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "2.6-PNR_add_multi_elements_response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "2.6-PNR_add_multi_elements-response", json_encode($responseArray)
            );

            return $responseArray;

        } catch (Exception $exception) {
            $e = "Error in PNR Add Multi Elements execute:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ['error' => $e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $now = Carbon::now();
        $formattedDate = $now->format('dmy');
        $formattedTime = $now->format('Hi');

        $xmlRequest = $this->xmlHeader($this->sequenceNumber, $this->amadeusSessionId, $this->amadeusSecurityToken) .
            "
            <soap:Body>

                  <PNR_AddMultiElements>
                      <pnrActions>
                        <optionCode>11</optionCode>
                      </pnrActions>
                      <dataElementsMaster>
                        <marker1 />

                        <dataElementsIndiv>
                          <elementManagementData>
                            <reference>
                              <qualifier>OT</qualifier>
                              <number>1</number>
                            </reference>
                            <segmentName>TK</segmentName>
                          </elementManagementData>
                          <ticketElement>
                            <ticket>
                              <indicator>XL</indicator>
                              <date>$formattedDate</date>
                              <time>$formattedTime</time>
                            </ticket>
                          </ticketElement>
                        </dataElementsIndiv>


                      </dataElementsMaster>
                    </PNR_AddMultiElements>


            </soap:Body>
        </soap:Envelope>
        ";

        self::storeInFile("xml", "2.6-PNR_add_multi_elements-request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/PNRADD_21_1_1A";
    }
}
