<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class QueryFareMasterPricerTravelBoardSearchAction extends AmadeusRequestStateless
{
    use FileStorageTrait;

    protected string $searchCriteriaString;
    protected string $passengerString;
    protected int $numberOfPassengers;
    protected array $searchCriteriaFlight;

    public function execute(array $passengerCriteria, array $searchCriteriaFlight, ?array $searchModifiers): array
    {

        try {
            $this->searchCriteriaFlight = $searchCriteriaFlight;
//            $this->searchCriteriaString = $this->generateSearchCriteria($searchCriteriaFlight);
            $this->passengerString = $this->generatePassengerCriteria($passengerCriteria);


            $response = $this->createCurlRequest();

            self::storeInFile("xml", "0-flight-search-response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "0-flight-search-response", json_encode($responseArray)
            );


            return $responseArray;

        } catch (\Exception $exception) {
            $e = "Error in FareInformativePricingAction:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ["error" => $e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $xml = $this->xmlHeader() . '
          <soap:Body>
            <Fare_MasterPricerTravelBoardSearch>
              <numberOfUnit>
                <unitNumberDetail>
                  <numberOfUnits>'.$this->getNumberOfUnitsForFareMasterPricer().'</numberOfUnits>
                  <typeOfUnit>RC</typeOfUnit>
                </unitNumberDetail>
                <unitNumberDetail>
                  <numberOfUnits>' . $this->numberOfPassengers . '</numberOfUnits>
                  <typeOfUnit>PX</typeOfUnit>
                </unitNumberDetail>
              </numberOfUnit>' .

            $this->passengerString

            . '
              <fareOptions>
                <pricingTickInfo>
                    <pricingTicketing>
                        <priceType>XND</priceType>
                        <priceType>ET</priceType>
                        <priceType>TAC</priceType>
                        <priceType>RP</priceType>
                        <priceType>IFS</priceType>
                        <priceType>RU</priceType>
                    </pricingTicketing>
                </pricingTickInfo>

                </fareOptions>

             ' .

            $this->generateSearchCriteria()

            . '</Fare_MasterPricerTravelBoardSearch>
          </soap:Body>

        </soap:Envelope>';

//        Log::info("SearchRequest: " . $xml);

        self::storeInFile("xml", "0-flight-search-request", $xml);

        return $xml;
    }

    protected function action(): string
    {
       return "http://webservices.amadeus.com/FMPTBQ_23_2_1A";
    }

    protected function generateSearchCriteria(): string
    {
        $xmlSearchNode = "";

        foreach ($this->searchCriteriaFlight as $index => $criteria) {
            list($from, $to, $departure) = $this->getSearchCriteria($criteria);

            $refNo = $index + 1;

            $xmlSearchNode .= "
                    <itinerary>
                        <requestedSegmentRef>
                          <segRef>$refNo</segRef>
                        </requestedSegmentRef>
                        <departureLocalization>
                          <departurePoint>
                            <locationId>$from</locationId>
                          </departurePoint>
                        </departureLocalization>
                        <arrivalLocalization>
                          <arrivalPointDetails>
                            <locationId>$to</locationId>
                          </arrivalPointDetails>
                        </arrivalLocalization>
                        <timeDetails>
                          <firstDateTimeDetail>
                            <date>$departure</date>
                          </firstDateTimeDetail>
                        </timeDetails>
                    </itinerary>
            ";
        }


        return $xmlSearchNode;
    }

    protected function getSearchCriteria(array $searchCriteriaFlight): array
    {
        $from = $searchCriteriaFlight['From']['value'];
        $to = $searchCriteriaFlight['To']['value'];
        $departureCarbonDate = Carbon::parse($searchCriteriaFlight['departureDate']);
        $departure = $departureCarbonDate->format("d") . $departureCarbonDate->format("m") . $departureCarbonDate->format("y");
        return array($from, $to, $departure);
    }

    protected function generatePassengerCriteria(array $passengerCriteria): string
    {
        $adt = 0;
        $chd = 0;
        $inf = 0;

        foreach ($passengerCriteria as $criterion) {
            if ($criterion['passengerTypeCode'] === "ADT") {
                $adt++;
            } else if ($criterion['passengerTypeCode'] === "CHD") {
                $chd++;

            } else {
                $inf++;
            }
        }

        $this->numberOfPassengers = $adt + $chd;

        $criteriaString = $this->createPTCWiseString($adt, "ADT", 1);

        if ($chd > 0) {
            $criteriaString .= $this->createPTCWiseString($chd, "CHD", $adt + 1);
        }

        if ($inf > 0) {
            $criteriaString .= $this->createInfantString($inf);
        }

        return $criteriaString;
    }

    protected function createPTCWiseString(int $passengerCount, string $ptc, int $startNumber): string
    {
        $s = "<paxReference><ptc>$ptc</ptc>";

        for ($x = $startNumber; $x < $passengerCount + $startNumber; $x++) {
            $s .= "<traveller><ref>$x</ref></traveller>";
        }

        $s .= "</paxReference>";

        return $s;
    }

    protected function createInfantString(int $infantCount): string
    {
        $s = "<paxReference><ptc>INF</ptc>";

        for ($x = 1; $x <= $infantCount; $x++) {
            $s .= "<traveller><ref>$x</ref><infantIndicator>1</infantIndicator></traveller>";
        }

        $s .= "</paxReference>";

        return $s;
    }
}
